package com.composrfoundation.codequalitychecker;

import java.awt.*;
import javax.swing.*;
import java.awt.datatransfer.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;

import java.util.*;
import java.io.*;

/**
 * Java Front-end to Web Code Quality Checker
 */
public class MainDialog extends JFrame {

    // UI

    JPanel panel1 = new JPanel();
    JButton scanFrontendBtn = new JButton();
    JButton scanBackendBtn = new JButton();
    JButton examineFilesBtn = new JButton();
    JButton forgetErrorBtn = new JButton();
    JButton viewCodeBtn = new JButton();
    JButton clearErrorsBtn = new JButton();
    JButton aboutBtn = new JButton();
    JButton optionsBtn = new JButton();
    JButton scanSignaturesBtn = new JButton();
    JButton countBtn = new JButton();
    JList files = null;
    JList errors = null;
    JLabel jLabel1 = new JLabel();
    JLabel jLabel2 = new JLabel();

    public MainDialog(String title) {
        this.setTitle(title);
        try {
            setDefaultCloseOperation(DISPOSE_ON_CLOSE);
            jbInit();
            pack();
        } catch (Exception exception) {
            exception.printStackTrace(System.out);
        }
    }

    public MainDialog() {
        this("Web Code Quality Checker");
    }

    private void jbInit() throws Exception {
        this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        this.setResizable(false);

        DefaultListModel model1 = new DefaultListModel();
        files = new JList(model1);
        DefaultListModel model2 = new DefaultListModel();
        errors = new JList(model2);
        JScrollPane scrollPaneFiles = new JScrollPane(files);
        JScrollPane scrollPaneErrors = new JScrollPane(errors);

        errors.setTransferHandler(new FSTransfer(this));
        errors.setDragEnabled(false);

        panel1.setLayout(null);
        scanFrontendBtn.setBounds(new Rectangle(8, 557, 107, 19));
        scanFrontendBtn.setMargin(new Insets(0, 0, 0, 0));
        scanFrontendBtn.setActionCommand("scanFrontendBtn");
        scanFrontendBtn.setText("<html>Frontend-scan</html>");
        scanFrontendBtn.addActionListener(new MainDialog_scanFrontendBtn_actionAdapter(this));
        scanFrontendBtn.setBackground(new Color(215, 245, 229));
        scanBackendBtn.setBounds(new Rectangle(8, 538, 107, 19));
        scanBackendBtn.setMargin(new Insets(0, 0, 0, 0));
        scanBackendBtn.setActionCommand("scanBackendBtn");
        scanBackendBtn.setText("<html>Backend-scan</html>");
        scanBackendBtn.addActionListener(new MainDialog_scanBackendBtn_actionAdapter(this));
        scanBackendBtn.setBackground(new Color(215, 245, 229));
        examineFilesBtn.setBounds(new Rectangle(194, 538, 78, 37));
        examineFilesBtn.setMargin(new Insets(0, 0, 0, 0));
        examineFilesBtn.setToolTipText("");
        examineFilesBtn.setActionCommand("examineFilesBtn");
        examineFilesBtn.setMnemonic('0');
        examineFilesBtn.setSelectedIcon(null);
        examineFilesBtn.setText("<html>Examine Selection</html>");
        examineFilesBtn.setBackground(new Color(215, 245, 229));
        examineFilesBtn.addActionListener(new MainDialog_examineFilesBtn_actionAdapter(this));
        forgetErrorBtn.setBounds(new Rectangle(504, 538, 58, 37));
        forgetErrorBtn.setMargin(new Insets(0, 0, 0, 0));
        forgetErrorBtn.setActionCommand("ForgetErrorBtn");
        forgetErrorBtn.setText("<html>Forget error</html>");
        forgetErrorBtn.setBackground(new Color(248, 247, 198));
        forgetErrorBtn.addActionListener(new MainDialog_ForgetErrorBtn_actionAdapter(this));
        viewCodeBtn.setBounds(new Rectangle(410, 538, 89, 37));
        viewCodeBtn.setMargin(new Insets(0, 0, 0, 0));
        viewCodeBtn.setActionCommand("ViewCodeBtn");
        viewCodeBtn.setText("<html>View code in editor</html>");
        viewCodeBtn.addActionListener(new MainDialog_ViewCodeBtn_actionAdapter(this));
        viewCodeBtn.setBackground(new Color(248, 247, 198));
        clearErrorsBtn.setBounds(new Rectangle(560, 538, 75, 37));
        clearErrorsBtn.setMargin(new Insets(0, 0, 0, 0));
        clearErrorsBtn.setActionCommand("ClearErrorsBtn");
        clearErrorsBtn.addActionListener(new MainDialog_ClearErrorsBtn_actionAdapter(this));
        clearErrorsBtn.setText("<html>Clear error list</html>");
        aboutBtn.setBounds(new Rectangle(644, 538, 65, 37));
        aboutBtn.setMargin(new Insets(0, 0, 0, 0));
        aboutBtn.setActionCommand("aboutBtn");
        aboutBtn.setText("<html>About</html>");
        aboutBtn.addActionListener(new MainDialog_aboutBtn_actionAdapter(this));
        optionsBtn.setBounds(new Rectangle(568, 538, 72, 37));
        optionsBtn.setMargin(new Insets(0, 0, 0, 0));
        optionsBtn.setActionCommand("optionsBtn");
        optionsBtn.setText("<html>Options</html>");
        optionsBtn.addActionListener(new MainDialog_optionsBtn_actionAdapter(this));
        scanSignaturesBtn.setBounds(new Rectangle(277, 538, 128, 37));
        scanSignaturesBtn.setMargin(new Insets(0, 0, 0, 0));
        scanSignaturesBtn.setActionCommand("scanSignaturesBtn");
        scanSignaturesBtn.setText("<html>Compile function signatures (PHP)</html>");
        scanSignaturesBtn.setBackground(new Color(215, 245, 229));
        scanSignaturesBtn.addActionListener(new MainDialog_scanSignaturesBtn_actionAdapter(this));
        countBtn.setBounds(new Rectangle(122, 538, 67, 37));
        countBtn.setMargin(new Insets(0, 0, 0, 0));
        countBtn.setActionCommand("countBtn");
        countBtn.setText("<html>Line Count</html>");
        countBtn.addActionListener(new MainDialog_countBtn_actionAdapter(this));
        countBtn.setBackground(new Color(215, 245, 229));
        panel1.setMinimumSize(new Dimension(790, 590));
        files.setBackground(new Color(215, 245, 229));
        files.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        files.addKeyListener(new MainDialog_files_actionAdapterKey(this));
        files.addMouseListener(new MainDialog_files_actionAdapterClick(this));
        scrollPaneFiles.setBounds(new Rectangle(10, 14, 389, 517));
        errors.setBackground(new Color(248, 247, 198));
        errors.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        errors.addKeyListener(new MainDialog_errors_actionAdapterKey(this));
        errors.addMouseListener(new MainDialog_errors_actionAdapterClick(this));
        scrollPaneErrors.setBounds(new Rectangle(406, 14, 885, 517));
        jLabel1.setToolTipText("");
        jLabel1.setText("<html>This is your project workspace. Set it by setting your code directory in the options.</html>");
        jLabel1.setBounds(new Rectangle(26, 490, 339, 37));
        jLabel2.setText(
            "<html>Either double click a file from the left, click examine selected "
            + "files, or just drag and drop files here from your OS.<br>"
            + "<br>Any errors will show up in this pane, and you\'ll be able "
            + "to view the involved code in the chosen editor via a double-click.</html>");
        jLabel2.setBounds(new Rectangle(415, 380, 350, 130));
        panel1.add(aboutBtn);
        panel1.add(countBtn);
        panel1.add(examineFilesBtn);
        panel1.add(scanSignaturesBtn);
        panel1.add(viewCodeBtn);
        panel1.add(forgetErrorBtn);
        //panel1.add(ClearErrorsBtn);
        panel1.add(optionsBtn);
        panel1.add(scanBackendBtn);
        panel1.add(scanFrontendBtn);
        panel1.add(jLabel1);
        panel1.add(scrollPaneFiles);
        panel1.add(jLabel2);
        panel1.add(scrollPaneErrors);
        this.getContentPane().add(panel1, java.awt.BorderLayout.WEST);
        panel1.setPreferredSize(new Dimension(1300, 590));

        this.setVisible(true);

        // Loading of file list
        EventQueue.invokeLater(scanBackendBtn::doClick);
    }

    public void initiateFileSearch(String type) {
        boolean sort_new = false, skip_custom = false;

        String path = Main.basePath.replace("\"", "");
        if ((path.equals(".")) || (path.equals("./")) || (path.equals(".\\"))) {
            path = "";
        }

        int n;
        n = JOptionPane.showOptionDialog(
            this,
            "Would you like to skip files underneath any directory with '*_custom' in the name?",
            "Question", JOptionPane.YES_NO_OPTION,
            JOptionPane.QUESTION_MESSAGE,
            null,
            null,
            null
        );
        if (n == JOptionPane.YES_OPTION) {
            skip_custom = true;
        }
        n = JOptionPane.showOptionDialog(
            this,
            "Would you like to sort by date?",
            "Question", JOptionPane.YES_NO_OPTION,
            JOptionPane.QUESTION_MESSAGE,
            null,
            null,
            null
        );
        if (n == JOptionPane.YES_OPTION) {
            sort_new = true;
        }

        ((DefaultListModel) this.files.getModel()).removeAllElements();

        java.util.List<String> directoriesToSkip = executePHPcode("include('" + path + "sources_custom/third_party_code.php'); echo implode(chr(10), list_untouchable_third_party_directories());");

        ArrayList<SearchFile> finalFiles = initiateFileSearch(type, path, "", skip_custom, directoriesToSkip);
        if (sort_new) {
            Collections.sort(finalFiles);
        }

        finalFiles.forEach((next) -> {
            ((DefaultListModel) this.files.getModel()).addElement(next.path);
        });
    }

    private ArrayList<SearchFile> initiateFileSearch(String type, String path, String rec_subpath, boolean skip_custom, java.util.List<String> directoriesToSkip) {
        Date d = new Date();
        long currentTime = d.getTime() / 1000;
        ArrayList<SearchFile> finalFiles = new ArrayList();

        File myFile = new File(path);
        String[] theFiles = myFile.list();
        if (theFiles == null) {
            JOptionPane.showMessageDialog(this, "Could not search the directory " + path + ".");
            return finalFiles;
        }
        Arrays.sort(theFiles);
        int i;
        long last_m;
        String relPath;
        File tmpFile;
        for (i = 0; i < theFiles.length; i++) {
            if (theFiles[i].equals(".") || theFiles[i].equals("..")) {
                continue;
            }

            relPath = ((rec_subpath.equals("")) ? "" : (rec_subpath.replace(File.separator, "/") + "/")) + theFiles[i];
            
            tmpFile = new File(path + File.separator + theFiles[i]);

            last_m = tmpFile.lastModified() / 1000 + 60 * 60 * 24;

            if (tmpFile.isDirectory()) {
                // Ignore floating cache/temporary stuff
                if ((theFiles[i].equals("_meta_tree"))
                        || (theFiles[i].equals("templates_cached"))) {
                    continue;
                }

                // third_party_code skipping
                if (directoriesToSkip.contains(relPath)) {
                    continue;
                }

                // Similar to IGNORE_NONBUNDLED (skips official non-bundled addons, rather than just third party code)
                if ((skip_custom) && (
                        (relPath.equals("uploads"))
                        || (relPath.equals("_tests"))
                        || (relPath.equals("mobiquo"))
                        || (relPath.equals("buildr"))
                        || (theFiles[i].contains("_custom")))) {
                    continue;
                }

                // Recurse
                ArrayList<SearchFile> next = initiateFileSearch(type, tmpFile.getAbsolutePath(), rec_subpath + ((rec_subpath.equals("")) ? "" : File.separator) + theFiles[i], skip_custom, directoriesToSkip);
                finalFiles.addAll(next);
            } else if (tmpFile.isFile()) {
                // Similar to IGNORE_SHIPPED_VOLATILE
                if (relPath.equals("_config.php") || relPath.equals("data_custom/errorlog.php") || relPath.equals("data_custom/execute_temp.php")) {
                    continue;
                }

                // Similar to IGNORE_ACCESS_CONTROLLERS
                if (tmpFile.length() == 0) {
                    continue;
                }

                 // Filter by file type
                if ((type.equals("Backend")) && (!theFiles[i].toLowerCase().endsWith(".php"))) {
                    continue;
                }
                if ((type.equals("Frontend"))
                        && (!theFiles[i].toLowerCase().endsWith(".css"))
                        && (!theFiles[i].toLowerCase().endsWith(".js"))
                        && (!theFiles[i].toLowerCase().endsWith(".html"))
                        && (!theFiles[i].toLowerCase().endsWith(".htm"))
                        && (!theFiles[i].endsWith(".tpl"))
                        && (!theFiles[i].toLowerCase().endsWith(".ini"))) {
                    continue;
                }

                // Add to file list
                SearchFile mySearchFile = new SearchFile(rec_subpath + ((rec_subpath.equals("")) ? "" : File.separator) + theFiles[i], tmpFile.lastModified());
                finalFiles.add(mySearchFile);
            }
        }

        return finalFiles;
    }

    public void aboutBtn_actionPerformed(ActionEvent e) {
        new AboutDialog().setVisible(true);
    }

    public void optionsBtn_actionPerformed(ActionEvent e) {
        new OptionsDialog().setVisible(true);
    }

    public void scanBackendBtn_actionPerformed(ActionEvent e) {
        initiateFileSearch("Backend");
    }

    public void scanSignaturesBtn_actionPerformed(ActionEvent e) {
        ((DefaultListModel) errors.getModel()).removeAllElements();
        executePHPfile("phpdoc_parser.php --base_path=" + Main.basePath.replace(" ", "\\ "));
    }

    public void scanFrontendBtn_actionPerformed(ActionEvent e) {
        initiateFileSearch("Frontend");
    }

    public void errors_actionPerformedKey(KeyEvent e) {
        if (e.getKeyChar() == '\n') {
            viewCodeLine();
        }
    }

    public void errors_actionPerformedMouse(MouseEvent e) {
        if ((e.getButton() == MouseEvent.BUTTON1) && (e.getClickCount() == 2)) {
            viewCodeLine();
        }
    }

    public void countBtn_actionPerformed(ActionEvent e) {
        int count = 0;
        int i;
        DefaultListModel listModel = (DefaultListModel) this.files.getModel();
        BufferedReader myReader;
        String line;
        for (i = 0; i < listModel.getSize(); i++) {
            if (!this.files.isSelectedIndex(i)) {
                continue;
            }

            try {
                myReader = new BufferedReader(new FileReader(Main.basePath + File.separator + (String) listModel.getElementAt(i)));
                line = myReader.readLine();
                while (line != null) {
                    if (!line.equals("")) {
                        count++;
                    }
                    line = myReader.readLine();
                }
            } catch (IOException e2) {
            } // No contribution to count
        }

        JOptionPane.showMessageDialog(this, "There are " + count + " lines of code in these files (excluding blank lines).");
    }

    private ArrayList<String> decompose_line(String line) {
        ArrayList<String> decomposed = new ArrayList<>();

        int i, num = 0;
        String current = "";
        boolean inquotes = false;
        String cchar;

        for (i = 0; i < line.length(); i++) {
            cchar = line.substring(i, i + 1);

            if (num == 4) {
                inquotes = true;
            }
            if ((cchar.equals("\"")) && (num < 4)) {
                inquotes = !inquotes;
            } else {
                if ((cchar.equals(" ")) && (!inquotes)) {
                    decomposed.add(current);
                    num++;
                    current = "";
                } else {
                    current = current + cchar;
                }
            }
        }
        decomposed.add(current);

        return decomposed;
    }

    private boolean line_skippable(String line) {
        ArrayList<String> decomposed = decompose_line(line);
        ArrayList<String> skip_decomposition;

        boolean same_0, same_1, same_3, same_4;
        int val_2_a, val_2_b;

        int i;
        for (i = 0; i < Main.skipped_errors.size(); i++) {
            skip_decomposition = decompose_line((String) Main.skipped_errors.get(i));

            if (decomposed.size() < 5) {
                continue;
            }

            same_0 = ((String) decomposed.get(0)).equals((String) skip_decomposition.get(0));
            same_1 = ((String) decomposed.get(1)).equals((String) skip_decomposition.get(1));
            same_3 = ((String) decomposed.get(3)).equals((String) skip_decomposition.get(3));
            same_4 = ((String) decomposed.get(4)).equals((String) skip_decomposition.get(4));
            try {
                val_2_a = Integer.parseInt((String) decomposed.get(2));
                val_2_b = Integer.parseInt((String) skip_decomposition.get(2));
                if ((same_0) && (same_1) && (val_2_a > val_2_b - 10) && (val_2_a < val_2_b + 10) && (same_3) && (same_4)) {
                    return true;
                }
            } catch (NumberFormatException e) {
            }
        }

        return false;
    }

    public void files_actionPerformedKey(KeyEvent e) {
        if (e.getKeyChar() == '\n') {
            ((DefaultListModel) errors.getModel()).removeAllElements();
            Object sv[] = new Object[1];
            sv[0] = files.getSelectedValue();
            do_execution(sv);
        }
    }

    public void files_actionPerformedMouse(MouseEvent e) {
        if ((e.getButton() == MouseEvent.BUTTON1) && (e.getClickCount() == 2)) {
            ((DefaultListModel) errors.getModel()).removeAllElements();
            Object sv[] = new Object[1];
            sv[0] = files.getSelectedValue();
            do_execution(sv);
        }
    }

    public void files_actionPerformed(ActionEvent e) {
        ((DefaultListModel) errors.getModel()).removeAllElements();
        Object[] sv = (Object[]) this.files.getSelectedValues();
        do_execution(sv);
    }

    public void do_execution(Object[] sv) {
        do_execution(sv, false);
    }

    public java.util.List<String> executePHPcode(String code) {
        String[] results = new String[0];
        ArrayList<String> cmdArray = new ArrayList<>();
        cmdArray.add(Main.phpPath);
        cmdArray.add("-r " + code);
        try {
            Process execution = Runtime.getRuntime().exec(cmdArray.toArray(new String[cmdArray.size()]));
            InputStream stream = execution.getInputStream();
            byte[] bytes = new byte[1024];
            String result = "";

            try {
                Thread.sleep(300);
            } catch (InterruptedException ex) {
            }

            int test = 0;
            while (test != -1) {
                test = stream.read(bytes);
                if (test != -1) {
                    result = result + new String(bytes, 0, test);
                }
            }
            results = result.split("\n");
        } catch (java.io.IOException e2) {
            JOptionPane.showMessageDialog(
                this,
                "Failure executing PHP backend. (" + e2.toString() + ")",
                "Error",
                JOptionPane.ERROR_MESSAGE
            );
        }

        return Arrays.asList(results);
    }

    public void executePHPfile(String line) {
        Dialog tempProgress = new ProcessingDialog();
        tempProgress.setVisible(true);

        line = Main.phpPath + " " + line;
        System.out.println(line);
        try {
            Process execution = Runtime.getRuntime().exec(line);
            InputStream stream = execution.getInputStream();
            byte[] bytes = new byte[1024];
            String result = "";

            try {
                Thread.sleep(300);
            } catch (InterruptedException ex) {
            }

            int test = 0;
            while (test != -1) {
                test = stream.read(bytes);
                if (test != -1) {
                    result = result + new String(bytes, 0, test);
                }
            }
            String[] results = result.split("\n");
            int j;
            for (j = 0; j < results.length; j++) {
                if (!line_skippable(results[j])) {
                    ((DefaultListModel) errors.getModel()).addElement(results[j]);
                }
            }
            this.jLabel1.setVisible(false);
            this.jLabel2.setVisible(false);
        } catch (java.io.IOException e2) {
            JOptionPane.showMessageDialog(
                this,
                "Failure executing PHP backend. (" + e2.toString() + ")",
                "Error",
                JOptionPane.ERROR_MESSAGE
            );
            tempProgress.setVisible(false);
            return;
        }

        tempProgress.setVisible(false);
    }

    public void do_execution(Object[] sv, boolean no_path) {
        int i, j;

        if (sv.length == 0) {
            JOptionPane.showMessageDialog(this, "No files were selected.", "Error", JOptionPane.ERROR_MESSAGE);
        }
        String line = "codechecker.php";
        if (!no_path) {
            line = line + " --base_path=" + Main.basePath.replace(" ", "\\ ");
        }
        if (Main.relay__api) {
            line = line + " --api";
        }
        if (Main.relay__todo) {
            line = line + " --todo";
        }
        if (Main.relay__mixed) {
            line = line + " --mixed";
        }
        if (Main.relay__pedantic) {
            line = line + " --pedantic";
        }
        if (Main.relay__somewhat_pedantic) {
            line = line + " --somewhat_pedantic";
        }
        if (Main.relay__security) {
            line = line + " --security";
        }
        if (Main.relay__manual_checks) {
            line = line + " --manual_checks";
        }
        if (Main.relay__spelling) {
            line = line + " --spelling";
        }
        if (Main.relay__codesniffer) {
            line = line + " --codesniffer";
        }
        if (Main.relay__eslint) {
            line = line + " --eslint";
        }
        for (i = 0; i < sv.length; i++) {
            line = line + " " + ((String) sv[i]).replace(" ", "\\ ");
        }

        executePHPfile(line);
    }

    private void viewCodeLine() {
        if (this.errors.getSelectedIndex() == -1) {
            JOptionPane.showMessageDialog(this, "No line was selected.");
            return;
        }

        String selected = (String) this.errors.getSelectedValue();

        ArrayList decomposed = decompose_line(selected);

        if ((decomposed.size() < 4)
                || ((!((String) decomposed.get(1)).endsWith(".php"))
                && (!((String) decomposed.get(1)).endsWith(".css"))
                && (!((String) decomposed.get(1)).endsWith(".js"))
                && (!((String) decomposed.get(1)).endsWith(".htm"))
                && (!((String) decomposed.get(1)).endsWith(".html"))
                && (!((String) decomposed.get(1)).endsWith(".tpl"))
                && (!((String) decomposed.get(1)).endsWith(".ini")))) {
            JOptionPane.showMessageDialog(this, "This line was not a code referencing line, so I cannot open up an editor there.");
            return;
        }

        String params;
        String line;
        String filePath = (((((String) decomposed.get(1)).charAt(1) == ':') || (((String) decomposed.get(1)).charAt(0) == '/')) ? "" : Main.basePath) + File.separator + decomposed.get(1);
        if (filePath.contains(" ")) {
            filePath = "\"" + filePath + "\"";
        }
        if (Main.textEditorPath.toLowerCase().endsWith("jedit.exe") || Main.textEditorPath.toLowerCase().endsWith("jedit")) {
            params = " +line:" + decomposed.get(2);
            line = Main.textEditorPath + " " + filePath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("notepad++.exe")) {
            params = " -multiInst " + filePath + " -n" + decomposed.get(2);
            line = Main.textEditorPath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("kate")) {
            params = " --line " + decomposed.get(2) + " --column " + decomposed.get(3);
            line = Main.textEditorPath + " " + filePath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("geany")) {
            params = " +" + decomposed.get(2);
            line = Main.textEditorPath + " " + filePath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("bbedit")) {
            params = " +" + decomposed.get(2);
            line = Main.textEditorPath + " " + filePath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("clphped.exe")) {
            params = " /g" + filePath + ":" + decomposed.get(2);
            line = Main.textEditorPath + " " + filePath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("codelobsteride.exe") || Main.textEditorPath.toLowerCase().endsWith("codelobsteride") || Main.textEditorPath.toLowerCase().endsWith("codelobster")) {
            params = ""; // TODO: Add line number support, once told about it
            line = Main.textEditorPath + " " + filePath + params;
        } else if ((Main.textEditorPath.toLowerCase().endsWith("netbeans.exe")) || (Main.textEditorPath.toLowerCase().endsWith("netbeans"))) {
            params = " --open " + filePath + ":" + decomposed.get(2);
            line = Main.textEditorPath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("mate")) {
            params = " -wl" + decomposed.get(2) + " " + filePath;
            line = Main.textEditorPath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("atom.exe") || Main.textEditorPath.toLowerCase().endsWith("atom.sh") || Main.textEditorPath.toLowerCase().endsWith("atom")) {
            params = ":" + decomposed.get(2);
            line = Main.textEditorPath + " " + filePath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("code.exe") || Main.textEditorPath.toLowerCase().endsWith("code")) {
            params = " -g " + filePath + ":" + decomposed.get(2);
            line = Main.textEditorPath + params;
        } else if (Main.textEditorPath.toLowerCase().endsWith("pspad.exe")) {
            params = " \\" + decomposed.get(2);
            line = Main.textEditorPath + params + " " + filePath;
        } else {
            params = "";
            line = Main.textEditorPath + " " + filePath + params;
        }

        if (this.errors.getSelectedIndex() == -1) {
            JOptionPane.showMessageDialog(this, "No file was selected.", "Error", JOptionPane.ERROR_MESSAGE);
        }
        try {
            Runtime.getRuntime().exec(line);
        } catch (java.io.IOException e) {
            JOptionPane.showMessageDialog(this, "Failure executing text editor.", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    public void ViewCodeBtn_actionPerformed(ActionEvent e) {
        viewCodeLine();
    }

    public void ForgetErrorBtn_actionPerformed(ActionEvent e) {
        Main.skipped_errors.add((String) errors.getSelectedValue());
        DefaultListModel model = (DefaultListModel) errors.getModel();
        model.remove(errors.getSelectedIndex());
        String writePath = "non_errors.txt";

        // Save skipped
        try {
            FileWriter writer = new FileWriter(writePath);
            try (PrintWriter out = new PrintWriter(writer)) {
                int i;
                for (i = 0; i < Main.skipped_errors.size(); i++) {
                    out.println((String) Main.skipped_errors.get(i));
                }
            }
        } catch (IOException e2) {
        } // No skip-saving then
    }
}

class MainDialog_errors_actionAdapterKey implements KeyListener {

    private final MainDialog adaptee;

    MainDialog_errors_actionAdapterKey(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void keyTyped(KeyEvent e) {
        adaptee.errors_actionPerformedKey(e);
    }

    @Override
    public void keyPressed(KeyEvent e) {
    }

    @Override
    public void keyReleased(KeyEvent e) {
    }
}

class MainDialog_errors_actionAdapterClick implements MouseListener {

    private final MainDialog adaptee;

    MainDialog_errors_actionAdapterClick(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void mouseClicked(MouseEvent e) {

        adaptee.errors_actionPerformedMouse(e);
    }

    @Override
    public void mouseEntered(MouseEvent e) {
    }

    @Override
    public void mouseExited(MouseEvent e) {
    }

    @Override
    public void mousePressed(MouseEvent e) {
    }

    @Override
    public void mouseReleased(MouseEvent e) {
    }
}

class MainDialog_files_actionAdapterKey implements KeyListener {

    private final MainDialog adaptee;

    MainDialog_files_actionAdapterKey(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void keyTyped(KeyEvent e) {
        adaptee.files_actionPerformedKey(e);
    }

    @Override
    public void keyPressed(KeyEvent e) {
    }

    @Override
    public void keyReleased(KeyEvent e) {
    }
}

class MainDialog_files_actionAdapterClick implements MouseListener {

    private final MainDialog adaptee;

    MainDialog_files_actionAdapterClick(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void mouseClicked(MouseEvent e) {

        adaptee.files_actionPerformedMouse(e);
    }

    @Override
    public void mouseEntered(MouseEvent e) {
    }

    @Override
    public void mouseExited(MouseEvent e) {
    }

    @Override
    public void mousePressed(MouseEvent e) {
    }

    @Override
    public void mouseReleased(MouseEvent e) {
    }
}

class MainDialog_ClearErrorsBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_ClearErrorsBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        ((DefaultListModel) adaptee.errors.getModel()).clear();
    }
}

class MainDialog_ForgetErrorBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_ForgetErrorBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        adaptee.ForgetErrorBtn_actionPerformed(e);
    }
}

class MainDialog_files_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_files_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        adaptee.files_actionPerformed(e);
    }
}

class MainDialog_ViewCodeBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_ViewCodeBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {

        adaptee.ViewCodeBtn_actionPerformed(e);
    }
}

class MainDialog_examineFilesBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_examineFilesBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {

        adaptee.files_actionPerformed(e);
    }
}

class MainDialog_countBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_countBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        adaptee.countBtn_actionPerformed(e);
    }
}

class MainDialog_scanFrontendBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_scanFrontendBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        adaptee.scanFrontendBtn_actionPerformed(e);
    }
}

class MainDialog_scanSignaturesBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_scanSignaturesBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        adaptee.scanSignaturesBtn_actionPerformed(e);
    }
}

class MainDialog_scanBackendBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_scanBackendBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        adaptee.scanBackendBtn_actionPerformed(e);
    }
}

class MainDialog_optionsBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_optionsBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        adaptee.optionsBtn_actionPerformed(e);
    }
}

class MainDialog_aboutBtn_actionAdapter implements ActionListener {

    private final MainDialog adaptee;

    MainDialog_aboutBtn_actionAdapter(MainDialog adaptee) {
        this.adaptee = adaptee;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        adaptee.aboutBtn_actionPerformed(e);
    }
}

class FSTransfer extends TransferHandler {

    private final MainDialog adaptee;

    public FSTransfer(MainDialog d) {
        this.adaptee = d;
    }

    @Override
    public boolean importData(JComponent comp, Transferable t) {
        // Make sure we have the right starting points
        if (!t.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
            return false;
        }

        // Grab the tree, its model and the root node
        try {
            java.util.List data = (java.util.List) t.getTransferData(DataFlavor.javaFileListFlavor);
            Object[] sl = data.toArray();
            if (adaptee != null) {
                String[] sl2 = new String[sl.length];
                int i;
                for (i = 0; i < sl.length; i++) {
                    sl2[i] = ((File) sl[i]).getAbsolutePath();
                }
                ((DefaultListModel) adaptee.errors.getModel()).
                        removeAllElements();
                adaptee.do_execution(sl2, true);
            }
            return true;
        } catch (UnsupportedFlavorException | IOException ufe) {
        }
        return false;
    }

    // We only support file lists on FSTrees...
    @Override
    public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
        for (DataFlavor transferFlavor : transferFlavors) {
            if (!transferFlavor.equals(DataFlavor.javaFileListFlavor)) {
                return false;
            }
        }
        return true;
    }
}
