<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    testing_platform
 */

/**
 * Composr test case class (unit testing).
 */
class __lang_spelling_epic_test_set extends cms_test_case
{
    public function testSpelling()
    {
        cms_extend_time_limit(TIME_LIMIT_EXTEND__CRAWL);
        disable_php_memory_limit();

        require_code('files2');
        require_code('spelling');

        require_once(get_file_base() . '/_tests/tests/async_tests/lang_spelling.php');
        $manual_check_ob = new lang_spelling_test_set();

        if (_find_spell_checker() === null) {
            $this->assertTrue(false, 'pspell/enchant required for this test');
            return;
        }

        // Many of these aren't real words, but they appear for good reasons so we safelist them
        $okay_words = [
            preg_replace('#\..*$#', '', get_domain()),
            'aaa',
            'escrowing',
            'escrowed',
            'escrowing',
            'flexform',
            'pax',
            'ipns',
            'passcode',
            'fscachepolicy',
            'pdts',
            'prioritising',
            'javanile',
            'adddefaultcharset',
            'deserialised',
            'filesmatch',
            'sudo',
            'unparsable',
            'ugo',
            'varchar',
            'chargelog',
            'misattributed',
            'lyndon',
            'tzinfo',
            'nullable',
            'foss',
            'dallas',
            'ua',
            'asd',
            'normalize',
            'xdiff',
            'toastify',
            'initialization',
            'initializing',
            'favor',
            'perl',
            'javaserver',
            'vbscript',
            'camelcase',
            'ecommerce',
            'ical',
            'oauth',
            'ocproducts',
            'oembed',
            'phpbb',
            'recaptcha',
            'samaccountname',
            'webfontloader',
            'ziparchive',
            'colouring',
            'webhook',
            'sequentiality',
            'gravatar',
            'gravatars',
            'occhris',
            'laa',
            'dipsy',
            'tinky',
            'winky',
            'grepimages',
            'composrcms',
            'stackexchange',
            'hybridauth',
            'freenode',
            'openid',
            'microsoftgraph',
            'cyrus',
            'ngrams',
            'ngram',
            'whacky',
            'falsy',
            'revolutionised',
            'visualisation',
            'fileowner',
            'filegroup',
            'acls',
            'vforum',
            'vtt',
            'firebase',
            'switcheroo',
            'prev',
            'perf',
            'kpis',
            'brotli',
            'durations',
            'analyzer',
            'ttl',
            'codesniffer',
            'goto',
            'jshint',
            'ide',
            'phpstan',
            'instanceof',
            'autoloading',
            'exakat',
            'phan',
            'unionised',
            'blobby',
            'reparsing',
            'centres',
            'munafo',
            'utomatically',
            'grawlixes',
            'boms',
            'endian',
            'intel',
            'wchars',
            'virtualbox',
            'codechecker',
            'roboto',
            'jwplayer',
            'bool',
            'focusin',
            'osano',
            'cdnjs',
            'fgetcsv',
            'techsmith',
            'libs',
            'cookieconsent',
            'navbar',
            'async',
            'galleryslideshowscreen',
            'calc',
            'mutationobserver',
            'mouseenter',
            'touchend',
            'playerstate',
            'fullscreen',
            'mr',
            'mlle',
            'sc',
            'csc',
            'mastercard',
            'bday',
            'impp',
            'goim',
            'screenname',
            'xmpp',
            'airplane',
            'grawlix',
            'umask',
            'changelogs',
            'rebase',
            'besluit',
            'colormind',
            'xfn',
            'organization', // Autofill name
            'punycode',
            'hreview',
            'upwork',
            'imagecopyresampled',
            'gitlab',
            'eol',
            'strcoll',
            'colspans',
            'ctype',
            'parenthesised',
            'fullname',
            'runpadcroptest',
            'exifrotated',
            'runexiftest',
            'runquadranttest',
            'toystory',
            'turkish',
            'eslintignore',
            'eslintrc',
            'yourbaseurl',
            'srand',
            'robotstxtscreen',
            'prefs',
            'msg',
            'anonymised',
            'dnt',
            'serialising',
            'anonymising',
            'afghanistan',
            'africa',
            'african',
            'albania',
            'alderney',
            'algeria',
            'andorra',
            'angola',
            'anguilla',
            'antarctica',
            'antigua',
            'antilles',
            'arab',
            'arabia',
            'argentina',
            'armenia',
            'aruba',
            'asia',
            'azerbaijan',
            'bahamas',
            'bahrain',
            'bangladesh',
            'barbados',
            'barbuda',
            'barth',
            'belarus',
            'belize',
            'benin',
            'bermuda',
            'bhutan',
            'bissau',
            'bolivia',
            'bosnia',
            'botswana',
            'bouvet',
            'brazil',
            'brunei',
            'bulgaria',
            'burkina',
            'burundi',
            'caicos',
            'caledonia',
            'cambodia',
            'cameroon',
            'cayman',
            'chile',
            'christmas',
            'colombia',
            'comoros',
            'congo',
            'costa',
            'croatia',
            'cuba',
            'cyprus',
            'czech',
            'dem',
            'denmark',
            'djibouti',
            'dominica',
            'dominican',
            'ecuador',
            'egypt',
            'eritrea',
            'estonia',
            'ethiopia',
            'falkland',
            'faroe',
            'faso',
            'fiji',
            'finland',
            'futuna',
            'gabon',
            'gambia',
            'georgia',
            'ghana',
            'gibraltar',
            'greenland',
            'grenada',
            'grenadines',
            'guadeloupe',
            'guam',
            'guatemala',
            'guernsey',
            'guiana',
            'guyana',
            'haiti',
            'helena',
            'herzegovina',
            'honduras',
            'hong',
            'hungary',
            'iceland',
            'indonesia',
            'iran',
            'iraq',
            'jamaica',
            'jordan',
            'kazakhstan',
            'kenya',
            'kiribati',
            'kitts',
            'kong',
            'korea',
            'kosovo',
            'kuwait',
            'kyrgyzstan',
            'lanka',
            'laos',
            'latvia',
            'lebanon',
            'lemy',
            'leone',
            'lesotho',
            'liberia',
            'libya',
            'liechtenstein',
            'lithuania',
            'lucia',
            'luxembourg',
            'macau',
            'macedonia',
            'madagascar',
            'malawi',
            'malaysia',
            'maldives',
            'mali',
            'malta',
            'mariana',
            'marino',
            'marshall',
            'martinique',
            'mauritania',
            'mauritius',
            'mayen',
            'mayotte',
            'mcdonald',
            'mexico',
            'micronesia',
            'miquelon',
            'moldova',
            'monaco',
            'mongolia',
            'montenegro',
            'montserrat',
            'mozambique',
            'myanmar',
            'namibia',
            'nauru',
            'ncipe',
            'nepal',
            'nevis',
            'nicaragua',
            'niger',
            'niue',
            'norfolk',
            'norway',
            'oceania',
            'oman',
            'openweathermap',
            'palau',
            'palestine',
            'papua',
            'paraguay',
            'peru',
            'philippines',
            'pierre',
            'pitcairn',
            'poland',
            'polynesia',
            'portugal',
            'puerto',
            'qatar',
            'rica',
            'rico',
            'romania',
            'rwanda',
            'sahara',
            'salvador',
            'samoa',
            'san',
            'saudi',
            'senegal',
            'seychelles',
            'singapore',
            'slovakia',
            'slovenia',
            'solomon',
            'somalia',
            'sri',
            'sudan',
            'suriname',
            'svalbard',
            'swaziland',
            'syria',
            'taiwan',
            'tajikistan',
            'tanzania',
            'thailand',
            'tobago',
            'togo',
            'tokelau',
            'tonga',
            'trinidad',
            'tunisia',
            'turkmenistan',
            'turks',
            'tuvalu',
            'uganda',
            'ukraine',
            'uruguay',
            'uzbekistan',
            'vanuatu',
            'vatican',
            'venezuela',
            'verde',
            'vietnam',
            'vincent',
            'wallis',
            'yemen',
            'zambia',
            'zealand',
            'zimbabwe',
            'pushtask',
            'cookieless',
            'textareacopycode',
            'gitstatus',
            'utilisation',
            'distro',
            'phpbeans',
            'javabeans',
            'systemd',
            'rsyslogd',
            'orm',
            'journald',
            'laravel',
            'conposr',
            'autofill',
            'autosuggest',
            'benchmarking',
            'bitstream',
            'canonicalisation',
            'collegehumor',
            'dejavu',
            'dvds',
            'freefont',
            'hdd',
            'imagettftext',
            'ish',
            'journaling',
            'newrelic',
            'swapspace',
            'xhprof',
            'linters',
            'fpm',
            'defs',
            'svgelementinstance',
            'unitless',
            'customproperty',
            'hasownproperty',
            'imageloadfont',
            'utopian',
            'autostart',
            'webvtt',
            'abbas',
            'bitwise',
            'eslint',
            'gruntfile',
            'markercluster',
            'markerclusterer',
            'mediaelementplayer',
            'uptodate',
            'ajaxify',
            'ajaxified',
            'regexes',
            'beforeunload',
            'modalwindow',
            'hasjsstate',
            'sexualised',
            'doxxing',
            'gunzip',
            'hrefurl',
            'bbedit',
            'codelobster',
            'minisiteinfo',
            'pspad',
            'cids',
            'cid',
            'chatjoin',
            'availablerelationship',
            'searchingpost',
            'browsedsupport',
            'darkplacement',
            'screenimages',
            'withconversr',
            'clipart',
            'debugfs',
            'readdir',
            'imaps',
            'imapt',
            'nocert',
            'nonmatch',
            'openbsd',
            'preferencing',
            'solaris',
            'plaintext',
            'starttls',
            'balancer',
            'usleep',
            'californian',
            'spectrums',
            'recompress',
            'inlined',
            'thefeedback',
            'ipstack',
            'svganimatedstring',
            'salman',
            'desimplifies',
            'harmlessurlcoder',
            'punycode',
            'appi',
            'currencyconverterapi',
            'domaintools',
            'ebashkoff',
            'getimagesize',
            'thumbnailing',
            'xf',
            'xffei',
            'xll',
            'xmm',
            'baidu',
            'bandcamp',
            'coinbase',
            'dtube',
            'ethereum',
            'exif',
            'flattr',
            'gamertag',
            'lastfm',
            'macos',
            'missingkeymaperror',
            'openbsd',
            'playstation',
            'qzone',
            'simplifier',
            'slashdot',
            'snapchat',
            'solaris',
            'spotify',
            'steemit',
            'tencent',
            'tieba',
            'unfixable',
            'viber',
            'wechat',
            'whatsapp',
            'zebadger',
            'staffpicks',
            'fffe',
            'appi',
            'currencyconverterapi',
            'domaintools',
            'ebashkoff',
            'getimagesize',
            'thumbnailing',
            'xf',
            'xffei',
            'xll',
            'xmm',
            'zebadger',
            'acknowledgement',
            'actualisation',
            'actualising',
            'addattachment',
            'analyse',
            'analysers',
            'analysing',
            'apologise',
            'artefacts',
            'authorisation',
            'authorisations',
            'authorise',
            'authorised',
            'authorising',
            'autoplay',
            'behaviour',
            'behaviours',
            'cancelled',
            'cancelling',
            'capitalisation',
            'capitalise',
            'capitalised',
            'catalogue',
            'catalogues',
            'categorisation',
            'categorisations',
            'categorise',
            'categorised',
            'categorises',
            'categorising',
            'centimetre',
            'centralise',
            'centralised',
            'centre',
            'centred',
            'centring',
            'cheque',
            'chequered',
            'cheques',
            'colour',
            'coloured',
            'colours',
            'contextualisation',
            'contextualised',
            'contextualises',
            'criticise',
            'criticising',
            'customisation',
            'customise',
            'customised',
            'customises',
            'customising',
            'decentralisation',
            'decentralised',
            'defence',
            'dependant',
            'disincentivise',
            'disincentivises',
            'empathise',
            'emphasised',
            'encyclopaedia',
            'encyclopaedic',
            'endeavour',
            'enquire',
            'equalise',
            'familiarise',
            'familiarising',
            'favour',
            'favouring',
            'favourite',
            'fieldname',
            'finalisation',
            'finalise',
            'finalised',
            'finalises',
            'formalised',
            'fulfil',
            'fulfilment',
            'fulfils',
            'funnelling',
            'generalise',
            'generalised',
            'globalisation',
            'globalise',
            'grey',
            'harbour',
            'harmonised',
            'honour',
            'honourable',
            'humour',
            'idealised',
            'immunisation',
            'immunisations',
            'immunised',
            'initialisation',
            'initialise',
            'initialised',
            'initialises',
            'initialising',
            'inserttextboxwysiwyg',
            'internalise',
            'internalised',
            'internationalisation',
            'internationalise',
            'internationalised',
            'isplaininsert',
            'italicise',
            'italicised',
            'itemised',
            'judgement',
            'labelled',
            'labelling',
            'labour',
            'learnt',
            'licence',
            'licences',
            'localisation',
            'localised',
            'maths',
            'maximise',
            'maximises',
            'maximising',
            'metres',
            'minimisation',
            'minimise',
            'minimised',
            'minimises',
            'misbehaviour',
            'modelling',
            'mould',
            'moulded',
            'neighbour',
            'neighbourhood',
            'neighbours',
            'normalisation',
            'normalise',
            'normalised',
            'offences',
            'optimisation',
            'optimisations',
            'optimise',
            'optimised',
            'optimiser',
            'optimises',
            'optimising',
            'organisation',
            'organisational',
            'organisations',
            'organise',
            'organised',
            'organises',
            'organising',
            'osm',
            'paedophile',
            'paedophiles',
            'penalise',
            'penalised',
            'personalise',
            'personalised',
            'plagiarise',
            'pluralisation',
            'pluralise',
            'postingfieldname',
            'practised',
            'prioritisation',
            'prioritise',
            'prioritised',
            'prioritises',
            'programmes',
            'randomisation',
            'randomise',
            'randomised',
            'randomising',
            'rationalise',
            'rationalised',
            'realisation',
            'realise',
            'realised',
            'realises',
            'realising',
            'reauthorise',
            'recognise',
            'recognised',
            'recognises',
            'recognising',
            'recoloured',
            'reinitialise',
            'reorganise',
            'reorganised',
            'reorganising',
            'saleable',
            'sanitise',
            'sanitised',
            'sanitising',
            'scrutinise',
            'scrutinised',
            'serialisation',
            'serialised',
            'setattachment',
            'signalling',
            'specialisation',
            'specialised',
            'specialising',
            'spelt',
            'standardisation',
            'standardise',
            'standardised',
            'standardising',
            'startnum',
            'stylise',
            'stylised',
            'subsidisation',
            'subsidise',
            'summarise',
            'summarised',
            'summarises',
            'symbolise',
            'synchronisation',
            'synchronise',
            'synchronised',
            'theatre',
            'travelling',
            'tunnelling',
            'unauthorised',
            'unlabelled',
            'unorganised',
            'unravelling',
            'unrecognisable',
            'unrecognised',
            'unrivalled',
            'uploadersettings',
            'utilise',
            'utilised',
            'utilises',
            'utilising',
            'victimised',
            'virtualisation',
            'visualise',
            'whizz',
            'wilfully',
            'aac',
            'abrupted',
            'absolute\'ing',
            'abusable',
            'acccounts',
            'accesscontrol',
            'accesskey',
            'accessor',
            'accountname',
            'acl',
            'acquia',
            'acteur',
            'actioned',
            'actioning',
            'actionlog',
            'actionlogs',
            'actualiser',
            'actualisers',
            'adblocker',
            'addon',
            'addons',
            'addonscreen',
            'addressbook',
            'addslashes',
            'adduser',
            'adg',
            'adipiscing',
            'admincp',
            'adminguide',
            'adminisative',
            'adminzone',
            'adsense',
            'adword',
            'adwords',
            'aef',
            'aefsid',
            'aenean',
            'afm',
            'afnetworking',
            'aging',
            'ahellobhellocdehellof',
            'ahref',
            'ai',
            'ajax',
            'al',
            'albert',
            'alberta',
            'albumname',
            'aliquam',
            'allen',
            'allocable',
            'allowlist',
            'america',
            'american',
            'americans',
            'amet',
            'ampm',
            'amsterdam',
            'analytics',
            'andd',
            'anonymisation',
            'anonymise',
            'anonymizers',
            'anselectcodelist',
            'answerer',
            'antialiasing',
            'antispam',
            'anyones',
            'apache',
            'apc',
            'apcu',
            'api',
            'apis',
            'apos',
            'appengine',
            'appfog',
            'appid',
            'appropiate',
            'approvers',
            'apr',
            'april',
            'arabic',
            'architected',
            'architecting',
            'arg',
            'argouml',
            'args',
            'aristotle',
            'armenian',
            'arthur',
            'articlesbase',
            'articletrader',
            'arvixe',
            'asc',
            'ascii',
            'ascript',
            'asf',
            'asort',
            'aspell',
            'assistive',
            'astrogrep',
            'atch',
            'atlassian',
            'atoz',
            'attachmentbrowsebutton',
            'attachmentsbrowser',
            'attainably',
            'atten',
            'auditable',
            'aug',
            'augue',
            'aui',
            'auiplugin',
            'australia',
            'austria',
            'auth',
            'auth\'d',
            'authorize',
            'authorized',
            'autoban',
            'autocomplete',
            'autocompleter',
            'autocompleting',
            'autocompletion',
            'autocorrected',
            'autodetect',
            'autodetected',
            'autodetection',
            'autofit',
            'autoformatting',
            'autogrow',
            'autoincrement',
            'autoload',
            'automatable',
            'autonomity',
            'autosave',
            'autosaving',
            'autosize',
            'avantlink',
            'avi',
            'axxxxx',
            'ay',
            'backburner',
            'backdoored',
            'backdoors',
            'backend',
            'backends',
            'backlinks',
            'backreference',
            'backrefs',
            'backslashed',
            'backslashing',
            'balkanisation',
            'balsamiq',
            'bannable',
            'bantr',
            'barnes',
            'basedir',
            'basename',
            'basepath',
            'bbcode',
            'bcc',
            'bcc\'d',
            'bcrypt',
            'behavior',
            'behaviors',
            'belgium',
            'beng',
            'benjamin',
            'besluit',
            'betterpopupblocker',
            'bg',
            'bgcolor',
            'bibendum',
            'bieber',
            'bigboy',
            'bing',
            'bitbucket',
            'bitkinex',
            'bitmask',
            'bitmasked',
            'bitnami',
            'bizarro',
            'blackbox',
            'blackhole',
            'blackholes',
            'blindheight',
            'blindwidth',
            'blitting',
            'bloaty',
            'blockmainactivitiesstate',
            'blockmainimagefader',
            'blockmainnotes',
            'blockmainsearch',
            'blockmainstafflinks',
            'blockquote',
            'blogpost',
            'blowfish',
            'bmp',
            'boh',
            'bonaparte',
            'bookables',
            'booker',
            'bookmarkable',
            'boolean',
            'booleans',
            'bootup',
            'bork',
            'boxify',
            'boxless',
            'br',
            'breakpoint',
            'bribable',
            'british',
            'browsable',
            'browserstack',
            'bruce',
            'bth',
            'btn',
            'buddha',
            'buddypress',
            'bugfix',
            'bugginess',
            'buildable',
            'buildout',
            'buildr',
            'bulleted',
            'buysellads',
            'buyspoons',
            'bys',
            'cacheable',
            'cachers',
            'cadorn',
            'california',
            'camden',
            'camtasia',
            'canada',
            'cancerresearch',
            'canonicalise',
            'canput',
            'captcha',
            'captchas',
            'cardinality',
            'caribbean',
            'carl',
            'carte',
            'casted',
            'catalogs',
            'catalogueaddingscreen',
            'catalogueeditingscreen',
            'caucho',
            'cc\'d',
            'ccbill',
            'cct',
            'cd',
            'cdn',
            'cdns',
            'cds',
            'celsius',
            'center',
            'centered',
            'centering',
            'centos',
            'centralized',
            'centric',
            'cgi',
            'chakra',
            'changelog',
            'charles',
            'charset',
            'charsets',
            'chatcode',
            'chatrooms',
            'chatroomscreen',
            'chdir',
            'cheaping',
            'checkboxes',
            'checkform',
            'checkmark',
            'childed',
            'chinese',
            'ching',
            'chmod',
            'chmodding',
            'chmoded',
            'chmoding',
            'chris',
            'chrisg',
            'christoph',
            'christophdorn',
            'chromagraphic',
            'churchhill',
            'circlesinside',
            'circlesrotate',
            'civicspace',
            'ckeditor',
            'clarke',
            'classnames',
            'cleanout',
            'clearfix',
            'clearstatcache',
            'clich',
            'clickability',
            'clickatell',
            'clickbank',
            'clickout',
            'clob',
            'closers',
            'cloudflare',
            'cloudinary',
            'cloudlinux',
            'cloudname',
            'cmd',
            'cms',
            'cmsgroups',
            'cmsnetworkmanager',
            'cmss',
            'cmsusers',
            'cn',
            'cname',
            'cnn',
            'cns',
            'cnsforumtopicwrapper',
            'cnsmemberprofilescreen',
            'codebase',
            'codebases',
            'codebook',
            'codebox',
            'codec',
            'codecs',
            'codelobster',
            'codename',
            'codenames',
            'codepoints',
            'coincidently',
            'collapsesource',
            'color',
            'colored',
            'colormind',
            'colors',
            'colourfy',
            'colspan',
            'columnisation',
            'columnise',
            'combinators',
            'combobox',
            'comcode',
            'comcodebigtabscontroller',
            'comcodemediaset',
            'comcodesectioncontroller',
            'cometchat',
            'commandr',
            'commentmeta',
            'commentspostingform',
            'commodo',
            'comparators',
            'compat',
            'complexed',
            'compliancy',
            'composr',
            'composrfoo',
            'composrto',
            'compsor',
            'computable',
            'conditionless',
            'conf',
            'config',
            'configurability',
            'configurational',
            'configurer',
            'conformability',
            'conformant',
            'congue',
            'connery',
            'consectetur',
            'consequat',
            'consequtive',
            'consuperrabiotriaxilator',
            'conversants',
            'conversr',
            'coolweens',
            'coords',
            'coppa',
            'copypasta\'d',
            'copywriting',
            'corel',
            'corporates',
            'corporatise',
            'corruptvalue',
            'costable',
            'coventry',
            'cpf',
            'cpfs',
            'cpu',
            'cpuinfo',
            'cqc',
            'cr',
            'creighton',
            'crit',
            'critera',
            'criterias',
            'cron',
            'cronjob',
            'crowdsource',
            'cruding',
            'crypto',
            'cryptographic',
            'cryptographically',
            'csp',
            'csrf',
            'css',
            'csv',
            'ctrl',
            'cubehide',
            'cubejelly',
            'cuberandom',
            'cubeshow',
            'cubesize',
            'cubespread',
            'cubestop',
            'cubestoprandom',
            'curation',
            'cursus',
            'customisability',
            'customisable',
            'customisations',
            'customize',
            'customized',
            'customprofilefields',
            'customware',
            'cvs',
            'cx',
            'cyber',
            'cyberduck',
            'cyclable',
            'cygwin',
            'cyrillic',
            'dai',
            'dailymotion',
            'dalton',
            'dapibus',
            'dat',
            'datalist',
            'datamappr',
            'dataset',
            'datastore',
            'date\'ll',
            'dateinterval',
            'datepicker',
            'datetime',
            'davfs',
            'david',
            'dba',
            'dbs',
            'ddthh',
            'de',
            'debian',
            'debrand',
            'debranding',
            'dec',
            'decache',
            'decached',
            'decaching',
            'december',
            'decentering',
            'decimalize',
            'declaratively',
            'decompositions',
            'decrement',
            'decrypt',
            'decrypted',
            'decrypting',
            'deduplicate',
            'defaultness',
            'defaultvalue',
            'defocus',
            'defocusing',
            'deinitiate',
            'deinvolvement',
            'del',
            'deletable',
            'delimite',
            //'demonstratr',
            'deployement',
            'deprioritise',
            'dereferenced',
            'descartes',
            'descendance',
            'descendents',
            'dest',
            'destrictify',
            'dev',
            'devguide',
            'devops',
            'devs',
            'dfn',
            'linkedin',
            'tumblr',
            'sina',
            'weibo',
            'vkontakte',
            'tencentweibo',
            'dh',
            'dialogs',
            'dicts',
            'dif',
            'differentiators',
            'diffmerge',
            'diffutils',
            'dignissim',
            'dir',
            'directadmin',
            'directionbottom',
            'directionleft',
            'directionright',
            'directiontop',
            'dirname',
            'dirs',
            'dirsimplify',
            'disableable',
            'disableregs',
            'disastr',
            'discernability',
            'discretionally',
            'dismissible',
            'dispatchment',
            'divs',
            'divx',
            'diyer',
            'diyers',
            'dkim',
            'dkimsignature',
            'dlink',
            'dload',
            'dlvr',
            'dn',
            'dnsbl',
            'dnsstuff',
            'doctype',
            'doctypes',
            'documentfragment',
            'doesn',
            'dolor',
            'dom',
            'domainkeys',
            'donald',
            'donec',
            'donext',
            'dont',
            'doored',
            'dorn',
            'dotnetnuke',
            'dotw',
            'dow',
            'downbars',
            'downloader',
            'dr',
            'dreamweaver',
            'dropbox',
            'dropdown',
            'dropdownmenu',
            'drupal',
            'dtango',
            'dublin',
            'dublincore',
            'duckduckgo',
            'duis',
            'dum',
            'dustme',
            'dyn',
            'dynamicness',
            'easter',
            'easyadd',
            'easycron',
            'easyphp',
            'ebooks',
            'ecards',
            'miswriting',
            'echosign',
            'ecmascript',
            'ecom',
            'edgehtml',
            'editability',
            'editarea',
            'editfrom',
            'efnet',
            'efnetrbl',
            'eg',
            'egestas',
            'eget',
            'einstein',
            'eithera',
            'el',
            'eleifend',
            'elementum',
            'elit',
            'elseif',
            'elses',
            'embeddable',
            'embedly',
            'empathic',
            'emtopx',
            'enableable',
            'enablement',
            'encodings',
            'encryptable',
            'endif',
            'england',
            'english',
            'englishenglish',
            'englishism',
            'enqueue',
            'ensim',
            'enterable',
            'entrancy',
            'enums',
            'env',
            'eot',
            'ephems',
            'equalities',
            'equivs',
            'erdos',
            'eros',
            'errflg',
            'erroring',
            'errorlog',
            'errormsg',
            'escaper',
            'escapeshellarg',
            'escapings',
            'esque',
            'et',
            'ethernet',
            'eu',
            'europe',
            'european',
            'eval',
            'eval\'d',
            'examp',
            'exe',
            'executables',
            'existency',
            'expander',
            'expanders',
            'expirations',
            'expiries',
            'explainer',
            'extendibility',
            'extentity',
            'extractive',
            'extranets',
            'extrasql',
            'extrovertism',
            'ezinearticles',
            'facebook',
            'facto',
            'fadefour',
            'fader',
            'fahrenheit',
            'failover',
            'failsafe',
            'fakepath',
            'fallbacks',
            'faq',
            'faqs',
            'farmville',
            'fastcgi',
            'favicon',
            'faviconist',
            'favicons',
            'favorite',
            'favorited',
            'favorites',
            'fb',
            'fbsr',
            'fclose',
            'feb',
            'februare',
            'february',
            'feedly',
            'feedreader',
            'felis',
            'femelle',
            'feugiat',
            'ffmpeg',
            'fi',
            'fieldid',
            'fieldmap',
            'fieldset',
            'filedump',
            'fileinode',
            'filepath',
            'filesize',
            'filesystem',
            'filesystems',
            'filetype',
            'filezilla',
            'filterability',
            'filtercode',
            'finalize',
            'findability',
            'findable',
            'fini',
            'finishprofile',
            'firebird',
            'firefox',
            'firephp',
            'firewalled',
            'firstname',
            'firsttime',
            'firstusername',
            'fixperms',
            'fixup',
            'fjs',
            'flexbox',
            'flic',
            'flickr',
            'flooders',
            'florida',
            'flushable',
            'fn',
            'fooing',
            'fopen',
            'foreach',
            'foremoreurl',
            'formorelead',
            'formoretext',
            'formoreurl',
            'formstandardend',
            'forumfoo',
            'forumid',
            'forumview',
            'forumviews',
            'forw',
            'fp',
            'frameborder',
            'frameset',
            'france',
            'francesco',
            'fread',
            'fred',
            'freetds',
            'freetype',
            'fri',
            'friday',
            'fringilla',
            'frontend',
            'frontpage',
            'fs',
            'fsockopen',
            'fstat',
            'fulltext',
            'func',
            'futurepay',
            'fwrite',
            'ga',
            'gae',
            'galeon',
            'gallerynav',
            'gandhi',
            'gantt',
            'gcd',
            'gd',
            'geany',
            'genfavicon',
            'geo',
            'geocode',
            'geocoding',
            'geoffrey',
            'geolocate',
            'geolocating',
            'geolocation',
            'geordi',
            'george',
            'georgian',
            'german',
            'germany',
            'geshi',
            'getelementsbytagname',
            'geterror',
            'getfieldnamesfortable',
            'getformvalues',
            'gethostbyaddr',
            'gethostbyname',
            'getresponsedata',
            'getter',
            'getuser',
            'gfx',
            'ghostbusters',
            'ghz',
            'gif',
            'gifs',
            'giftr',
            'gimpshop',
            'github',
            'tiktok',
            'gitlab',
            'givenmatch',
            'glassblock',
            'glasscube',
            'global\'d',
            'globalhelperpanel',
            'globals',
            'glockapps',
            'gmail',
            'gnudist',
            'googleapis',
            'googlebot',
            'gotama',
            'gray',
            'greece',
            'greek',
            'greenpeace',
            'gregorian',
            'gretel',
            'groups\'so',
            'grr',
            'gsutil',
            'guerilla',
            'guid',
            'guids',
            'gustav',
            'gz',
            'gzip',
            'gzipped',
            'gzipping',
            'hackable',
            'hackattack',
            'hackerish',
            'hacky',
            'halloween',
            'hander',
            'handleable',
            'hansel',
            'hardcode',
            'hardcoded',
            'hardcoding',
            'hasproperty',
            'haved',
            'hc',
            'hcalendar',
            'hcard',
            'hdf',
            'healthpersonalcare',
            'hehe',
            'heidisql',
            'helpdesk',
            'helptheaged',
            'hemorrhoids',
            'hendrerit',
            'henry',
            'heredoc',
            'heroku',
            'heros',
            'hh',
            'hidebars',
            'highrange',
            'hinglish',
            'hipchat',
            'hiphop',
            'hippyvm',
            'hoc',
            'holleywoodstudio',
            'holmes',
            'homebrew',
            'homegarden',
            'homesite',
            'honor',
            'horiz',
            'hostname',
            'hostnames',
            'hotfix',
            'hotfixes',
            'hoth',
            'hotspots',
            'hotword',
            'hqdefault',
            'href',
            'hsv',
            'htaccess',
            'htauth',
            'htdocs',
            'htm',
            'html',
            'htmlelement',
            'htmlentities',
            'htmlspecialchars',
            'http',
            'httpauth',
            'httpbl',
            'httpdocs',
            'https',
            'hunspell',
            'hyphenised',
            'i\'d',
            'i\'ll',
            'i\'m',
            'i\'ve',
            'iaas',
            'iaculis',
            'iceweasel',
            'iconv',
            'ics',
            'idd',
            'ident',
            'idevices',
            'ie',
            'ietester',
            'ified',
            'iframe',
            'iframed',
            'iframely',
            'iframes',
            'iis',
            'im',
            'imagecopyresized',
            'imagecreatefromwebp',
            'imagefilledrectangle',
            'hookless',
            'imagecreatefrombmp',
            'imagemagick',
            'imageoptim',
            'imagettfbbox',
            'imaginery',
            'imap',
            'imdb',
            'img',
            'imperdiet',
            'importances',
            'ims',
            'incase',
            'incentivisation',
            'incentivise',
            'incentivised',
            'incentivising',
            'include\'d',
            'incompliant',
            'incrementing',
            'indexable',
            'india',
            'indian',
            'indiscriminatingly',
            'infowindow',
            'infozip',
            'ing',
            'ini',
            'init',
            'initialisations',
            'initialiser',
            'initialize',
            'initialized',
            'initializedatabase',
            'inlining',
            'innodb',
            'innokenty',
            'inputed',
            'inputter',
            'inputters',
            'insertbefore',
            'insta',
            'instagram',
            'tiktok',
            'gitlab',
            'installable',
            'installatron',
            'installdir',
            'installprofile',
            'instantiation',
            'instantiations',
            'integrations',
            'integrators',
            'interbase',
            'interdependencies',
            'interoperability',
            'interoperate',
            'interprocess',
            'interstitially',
            'intervaling',
            'intl',
            'intra',
            'intval',
            'intval\'d',
            'invision',
            'inviter',
            'io',
            'iodbc',
            'ip',
            'ipall',
            'ipb',
            'iphones',
            'ipod',
            'ips',
            'ipso',
            'ipsum',
            'ireland',
            'isaacson',
            'isbn',
            'ish',
            'isnotspam',
            'ispconfig',
            'isps',
            'israel',
            'isrequired',
            'isresponsevalid',
            'isset',
            'italian',
            'italy',
            'itemprop',
            'iterable',
            'iteratively',
            'itunes',
            'jailbreakers',
            'jan',
            'jane',
            'januare',
            'january',
            'javacript',
            'javadoc',
            'javascript',
            'javascriptcore',
            'javascripts',
            'jedit',
            'jestr',
            'jetbrains',
            'jewelry',
            'jiggery',
            'jim',
            'jing',
            'jitsi',
            'johndoe',
            'johnson',
            'joindate',
            'jpeg',
            'jpegs',
            'jpg',
            'js',
            'jsdoc',
            'json',
            'jsonview',
            'judgment',
            'jul',
            'julie',
            'july',
            'june',
            'jung',
            'justin',
            'ka',
            'kama',
            'kate',
            'kbd',
            'kerberos',
            'keydown',
            'keyframe',
            'keymap',
            'keymatches',
            'keypress',
            'keypresses',
            'keyset',
            'kib',
            'kindlestore',
            'kompare',
            'konqueror',
            'kr',
            'ksort',
            'kwaliteit',
            'kwrite',
            'labeled',
            'lacus',
            'lang',
            'langage',
            'langfilter',
            'langs',
            'lastname',
            'lastness',
            'lastondate',
            'lasttime',
            'lastusername',
            'latfield',
            'latin',
            'layed',
            'lazenby',
            'lc',
            'lcs',
            'ldap',
            'le',
            'leaderboard',
            'lectus',
            'leet',
            'leo',
            'leste',
            'levenshtein',
            'lex',
            'lexed',
            'lexing',
            'lhs',
            'li',
            'libreoffice',
            'lifecycle',
            'lightbox',
            'lightboxes',
            'limit\'ing',
            'linearised',
            'luminance',
            'gaussian',
            'tu',
            'linearise',
            'linearize',
            'linearized',
            'linebreak',
            'linebreaks',
            'linkable',
            'linkedin',
            'linkers',
            'linkhtml',
            'linklist',
            'linkshare',
            'linksyntax',
            'linux',
            'litespeed',
            'loadaverage',
            'lobortis',
            'loc',
            'localhost',
            'lockdown',
            'logics',
            'loginkey',
            'logitude',
            'logpass',
            'loguid',
            'lol',
            'lon',
            'london',
            'longdesc',
            'longfield',
            'longtext',
            'longtitude',
            'lookahead',
            'lookups',
            'loopback',
            'lorem',
            'lossy',
            'lowrange',
            'lt',
            'ltr',
            'luther',
            'lve',
            'ly',
            'macintosh',
            'maecenas',
            'magento',
            'magna',
            'mailchimp',
            'mailtester',
            'mailto',
            'malloc',
            'manhattan',
            'mantisbt',
            'mapquest',
            'margining',
            'mariadb',
            'markable',
            'massa',
            'masse',
            'matchkey',
            'matermarks',
            'math',
            'matsushita',
            'mattis',
            'maxheight',
            'maxima',
            'maxresdefault',
            'maxs',
            'maxwidth',
            'mbs',
            'mbstring',
            'md',
            'measureit',
            'mediaelement',
            'mediawiki',
            'memberid',
            'memcache',
            'memcached',
            'memtrack',
            'mensa',
            'mergeable',
            'messagereturned',
            'metapad',
            'metus',
            'mhash',
            'microdata',
            'microformats',
            'microsite',
            'microsoft',
            'middot',
            'midrange',
            'mighr',
            'mimetype',
            'mingw',
            'miniblock',
            'minification',
            'minified',
            'minify',
            'minikernel',
            'minimalise',
            'minimalistic',
            'minimodule',
            'minishell',
            'miniview',
            'mins',
            'mis',
            'misassociated',
            'miscommunicated',
            'misconfiguration',
            'misconfigured',
            'misrouted',
            'misset',
            'mistakingly',
            'mistyped',
            'mitigations',
            'mizner',
            'mktime',
            'mobilemenu',
            'mobiquo',
            'mockup',
            'modability',
            'modally',
            'modcp',
            'moderations',
            'modsecurity',
            'modularisation',
            'modularisations',
            'modularise',
            'modularised',
            'modularity',
            'modularly',
            'moduli',
            'mohandas',
            'molestie',
            'mollis',
            'mon',
            'monday',
            'monikerified',
            'monikerised',
            'monitonic',
            'monoculture',
            'monospaced',
            'moono',
            'moonocolor',
            'moore',
            'moov',
            'mousemove',
            'mouseout',
            'mouseover',
            'mouseup',
            'mov',
            'moz',
            'mozilla',
            'mqdefault',
            'msappgallery',
            'msn',
            'mstate',
            'msvideo',
            'mtime',
            'mtimes',
            'muhahahahahaha',
            'multibyte',
            'multiline',
            'multilist',
            'multipart',
            'multiplier',
            'multisite',
            'multiword',
            'munges',
            'musicalinstruments',
            'mvc',
            'mw',
            'mx',
            'mybb',
            'mybbuser',
            'mycomputer',
            'mycustomstyle',
            'mycustomtemplate',
            'mydomain',
            'myfile',
            'myisam',
            'myname',
            'mypage',
            'mysite',
            'mysitetheme',
            'mysql',
            'mysqli',
            'mysqls',
            'myvideostyle',
            'myvideosyle',
            'mywork',
            'na',
            'nameserver',
            'nameservers',
            'namespace',
            'namespaces',
            'namings',
            'nan',
            'narrowin',
            'natively',
            'nav',
            'navigations',
            'nb',
            'nbsp',
            'nce',
            'ndash',
            'nec',
            'neque',
            'nestings',
            'netbeans',
            'netdrive',
            'netherhope',
            'netherlands',
            'netlink',
            'netscape',
            'newcode',
            'newroom',
            'nginx',
            'nibh',
            'nich',
            'nigeria',
            'nike',
            'nisi',
            'nl',
            'nm',
            'nodelay',
            'noembed',
            'nofollow',
            'noindex',
            'nokia',
            'nominet',
            'nonces',
            'noncharacter',
            'nondeterministic',
            'nones',
            'noop',
            'noreply',
            'noscript',
            'nother',
            'nothumbplayer',
            'notificationbuttons',
            'notificatons',
            'notifydevicetokentoserver',
            'nots',
            'nov',
            'november',
            'nowrap',
            'nulla',
            'nullability',
            'nullam',
            'num',
            'numberinput',
            'numval',
            'nunc',
            'nusphere',
            'oauth\'d',
            'obb',
            'oci',
            'ocp',
            'ocproduct',
            'ocptutor',
            'oct',
            'october',
            'octocat',
            'odbc',
            'offical',
            'officeproducts',
            'ogg',
            'ohloh',
            'ok',
            'ol',
            'olson',
            'omni',
            'omniweb',
            'on\'t',
            'onchange',
            'onclick',
            'ondragend',
            'onhold',
            'onkeypress',
            'onload',
            'onsubmit',
            'onwards',
            'opcache',
            'opendkim',
            'opendmoz',
            'openldap',
            'openoffice',
            'opensearch',
            'opensearchdescription',
            'openshift',
            'openssl',
            'openstack',
            'openstreetmap',
            'operability',
            'oppenheimer',
            'optgroup',
            'optimi',
            'optimisers',
            'optimization',
            'optimize',
            'optimizely',
            'optimizer',
            'or\'d',
            'orci',
            'orderby',
            'orderer',
            'organize',
            'oring',
            'ornare',
            'os',
            'ots',
            'ou',
            'ourself',
            'outerhtml',
            'outmode',
            'outmoding',
            'overcomplicate',
            'overdefined',
            'overlongs',
            'overridable',
            'ownerships',
            'oxfam',
            'paas',
            'pagename',
            'pagerank',
            'pagespeed',
            'paginations',
            'pakistan',
            'pam',
            'pandoras',
            'paralell',
            'parallelisation',
            'param',
            'parameterisation',
            'parameterised',
            'paramname',
            'params',
            'parentnode',
            'parsability',
            'parsedevicetoken',
            'participative',
            'passworded',
            'passwordless',
            'pasteable',
            'pathname',
            'patreon',
            'patreons',
            'paul',
            'paypal',
            'pc',
            'pch',
            'pchardware',
            'pdf',
            'pdfs',
            'peachpie',
            'pedia',
            'pellentesque',
            'pem',
            'performant',
            'personalisation',
            'petsupplies',
            'phablets',
            'phasellus',
            'philip',
            'photobucket',
            'photoshop',
            'photoshops',
            'photoupload',
            'php',
            'phpdoc',
            'phpdocumentor',
            'phpexpress',
            'phphelp',
            'phpinfo',
            'phpmyadmin',
            'phpnuke',
            'phps',
            'phpsettings',
            'phpstorm',
            'phpstub',
            'phpversion',
            'pickpocketed',
            'pictographic',
            'piglatin',
            'pinf',
            'pinterest',
            'playerheight',
            'playeroptions',
            'playerwidth',
            'playlistitems',
            'plesk',
            'plowing',
            'plugable',
            'plupload',
            'pms',
            'png',
            'pnggauntlet',
            'pngs',
            'pngthing',
            'po',
            'pokery',
            'policier',
            'pollbox',
            'polyfill',
            'polyfills',
            'pomme',
            'popupmenu',
            'popupmenubranch',
            'porttitor',
            'pos',
            'postgres',
            'postgresql',
            'postmeta',
            'postsize',
            'pps',
            'pre',
            'preagreed',
            'preallocated',
            'prebuilt',
            'precache',
            'precalculated',
            'precedences',
            'precheck',
            'precompiled',
            'precomposited',
            'precreated',
            'predefine',
            'preg',
            'preinstalled',
            'preload',
            'preloaded',
            'preloading',
            'premade',
            'prepend',
            'prepended',
            'prepending',
            'prepopulate',
            'prepopulated',
            'prepopulates',
            'prepopulating',
            'preprocess',
            'preprocessable',
            'preprocessed',
            'preprocessing',
            'preprocessor',
            'prespecified',
            'pretranslated',
            'prettyname',
            'prewritten',
            'pricings',
            'probations',
            'proc',
            'procedurally',
            'processables',
            'profiler',
            'programmatically',
            'promotable',
            'proxied',
            'proxying',
            'proxys',
            'psd',
            'pspell',
            'pte',
            'pts',
            'pulvinar',
            'punctuationless',
            'punisher',
            'purchasewizardscreen',
            'pushstate',
            'pwd',
            'pwds',
            'px',
            'qa',
            'qbbr',
            'quercus',
            'queryable',
            'queueing',
            'quicktime',
            'quis',
            'quisque',
            'quora',
            'rackspace',
            'radionodelist',
            'rakuten',
            'randombits',
            'randomsmart',
            'raquo',
            'rateable',
            'rawurldecode',
            'rawurlencode',
            'rb',
            'rbl',
            'rbls',
            'rcpthosts',
            'rdf',
            'rdn',
            'readlogs',
            'readme',
            'readonly',
            'reallocations',
            'realmedia',
            'realplayer',
            'realtime',
            'rearchitect',
            'rearchitected',
            'reasonings',
            'reassociates',
            'reattemption',
            'rebalances',
            'rebranded',
            'rebranding',
            'recached',
            'recalc',
            'recency',
            'recommender',
            'reconfigurations',
            'reconfiguring',
            'recurse',
            'recursed',
            'recursing',
            'reddit',
            'redhat',
            'redirections',
            'reebok',
            'reely',
            'reenable',
            'reentrance',
            'referer',
            'referers',
            'reflowing',
            'regionalisation',
            'regionalities',
            'regionality',
            'reimport',
            'reimporting',
            'reindex',
            'reinitiated',
            'reinstallation',
            'reinstalls',
            'reinvite',
            'relatin',
            'remapper',
            'remappings',
            'renderable',
            'renderer',
            'renderers',
            'rene',
            'reparse',
            'repimages',
            'replacer',
            'replicable',
            'reputationenable',
            'reqs',
            'requestor',
            'require\'d',
            'resample',
            'resampling',
            'resave',
            'rescale',
            'reselect',
            'reseller',
            'resellers',
            'resending',
            'resetdimensions',
            'reshow',
            'resizable',
            'resizer',
            'restorable',
            'restrictify',
            'reupload',
            'reuploading',
            'reusability',
            'revalidated',
            'reversable',
            'rewritebase',
            'rewritemap',
            'rhoncus',
            'rhs',
            'rigourous',
            'rijksoverheidswebsites',
            'risus',
            'rmdir',
            'roadmap',
            'roadsend',
            'robert',
            'robo',
            'robocop',
            'rome',
            'rong',
            'roomname',
            'routable',
            'rsaquo',
            'rsquo',
            'rss',
            'rsync',
            'rtl',
            'runnable',
            'runtime',
            'russell',
            'russia',
            'russian',
            'rwx',
            'sa',
            'sabredav',
            'sagittis',
            'saidless',
            'sajaxupdatelocking',
            'salesy',
            'samp',
            'samuel',
            'sandboxed',
            'sandboxing',
            'sanitisation',
            'sanitize',
            'santisations',
            'sapien',
            'saturday',
            'scalability',
            'scalable',
            'scancelled',
            'scelerisque',
            'schemas',
            'scotty',
            'screencast',
            'screencasts',
            'screenflow',
            'scribd',
            'scriptscript',
            'scrollable',
            'scrollbar',
            'scrollbars',
            'scroller',
            'scrutinized',
            'sddefault',
            'sdk',
            'sdks',
            'se',
            'seach',
            'seamonkey',
            'sean',
            'searc',
            'searchable',
            'searchformscreen',
            'securetransport',
            'securitylogging',
            'sed',
            'segoe',
            'selectable',
            'selectcode',
            'selectlist',
            'selectmenu',
            'selinux',
            'selinuxfs',
            'semihtml',
            'semiparse',
            'semper',
            'semrush',
            'sendmail',
            'seo',
            'sep',
            'september',
            'seqs',
            'serbia',
            'serialiser',
            'serialization',
            'serialize',
            'serialized',
            'serps',
            'serverid',
            'servername',
            'sestatus',
            'setcookie',
            'seting',
            'setness',
            'setupwizard',
            'sharding',
            'shareasale',
            'shareddemo',
            'shatterer',
            'shcore',
            'sheffield',
            'sherlock',
            'shippable',
            'shippo',
            'shish',
            'shoutbox',
            'shoutboxes',
            'showbars',
            'showbarsrandom',
            'showloader',
            'shownotification',
            'sig',
            'signup',
            'signups',
            'sigs',
            'silktide',
            'siloed',
            'simpletest',
            'simplexmlelement',
            'sitelock',
            'sitewide',
            'situ',
            'sizeness',
            'sizings',
            'skillsets',
            'skippable',
            'sku',
            'skype',
            'slideshare',
            'smallprint',
            'smartbanner',
            'smartftp',
            'smf',
            'smfads',
            'smilies',
            'smodified',
            'sms',
            'smtp',
            'sn',
            'snapback',
            'soapclient',
            'softaculous',
            'sokolov',
            'soletary',
            'somethingwebsystem',
            'someuser',
            'sortable',
            'sortablefield',
            'soundcloud',
            'sourceless',
            'spacings',
            'spain',
            'spamassassin',
            'spambots',
            'spamlist',
            'spammy',
            'spec\'d',
            'spidering',
            'spidermonkey',
            'sponsorships',
            'sportinggoods',
            'sql',
            'sqlite',
            'sqlserver',
            'sqlsrv',
            'sr',
            'src',
            'ss',
            'ssds',
            'ssl',
            'ssw',
            'startpt',
            'stateful',
            'staticflickr',
            'stderr',
            'stdin',
            'stdout',
            'stealr',
            'stemmer',
            'steve',
            'steven',
            'stickied',
            'stildv',
            'stopforumspam',
            'stopword',
            'str',
            'strategise',
            'stringwithformat',
            'strippable',
            'strlen',
            'strpos',
            'strtolower',
            'strtotime',
            'strtoupper',
            'strval',
            'stucked',
            'stylesheet',
            'stylesheets',
            'stylings',
            'stylize',
            'su',
            'subaccount',
            'subcat',
            'subcats',
            'subclasses',
            'subcom',
            'subcommunities',
            'subcommunity',
            'subdir',
            'subdirectories',
            'subdirectory',
            'subdirs',
            'subdomain',
            'subdomains',
            'subfolder',
            'subfolders',
            'subforum',
            'subforums',
            'subgalleries',
            'subgallery',
            'submenu',
            'submittable',
            'submitters',
            'suboptimal',
            'subpage',
            'subpath',
            'subpaths',
            'subproblems',
            'subquery',
            'subrecords',
            'subselect',
            'subsite',
            'subsites',
            'subst',
            'substr',
            'substring',
            'substrings',
            'subsubforum',
            'subsubforums',
            'subsyntax',
            'subtab',
            'subtree',
            'subtrees',
            'subview',
            'sucuri',
            'sugarcrm',
            'suhosin',
            'suitecrm',
            'sunday',
            'superclass',
            'supermoderators',
            'superset',
            'surveymonkey',
            'svg',
            'swapbars',
            'swapbarsback',
            'swapblocks',
            'sweden',
            'switzerland',
            'sybase',
            'symlink',
            'symlinked',
            'symlinking',
            'symlinks',
            'synched',
            'synching',
            'synchronize',
            'syndications',
            'syntaxes',
            'syntaxhighlighter',
            'sys',
            'sz',
            'tabindexes',
            'tada',
            'taglessly',
            'tahoma',
            'taintable',
            'tamil',
            'tapatalk',
            'tapatalkpush',
            'taskbar',
            'taxcloud',
            'taxjar',
            'tcp',
            'td',
            'tds',
            'techcrunch',
            'teletyped',
            'tellus',
            'tempcode',
            'templated',
            'templating',
            'tempnam',
            'terre',
            'tesco',
            'testability',
            'texas',
            'textarea',
            'textcode',
            'textedit',
            'textfield',
            'textmate',
            'tf',
            'th',
            'thay',
            'thead',
            'thead\'d',
            'themegen',
            'themeing',
            'themer',
            'themers',
            'themetemplateeditortab',
            'themewizard',
            'theming',
            'there\'d',
            'there\'ll',
            'thomas',
            'thoreau',
            'threadid',
            'thu',
            'thumbimg',
            'thunderbird',
            'thursday',
            'thusly',
            'tickbox',
            'tid',
            'tiering',
            'tim',
            'timein',
            'timelimit',
            'timezone\'so',
            'timezoned',
            'timezones',
            'timor',
            'tincidunt',
            'tinyportal',
            'titlesdata',
            'tmp',
            'toc',
            'todo',
            'togabd',
            'toggleable',
            'toggleabletray',
            'tokbox',
            'tokenisation',
            'tokenise',
            'tooltip',
            'tooltips',
            'topicview',
            'tornados',
            'tornevall',
            'tornevallrbl',
            'tornuserinfoclass',
            'tortor',
            'tpl',
            'traceability',
            'tracert',
            'trackback',
            'trackbacked',
            'trackbacks',
            'trak',
            'transactional',
            'transcode',
            'transcoded',
            'transcoder',
            'transcoders',
            'transcoding',
            'transifex',
            'treedata',
            'treelist',
            'treemenu',
            'treturn',
            'tricksy',
            'trigered',
            'tristique',
            'trojan',
            'trs',
            'truetype',
            'truey',
            'tt',
            'tucows',
            'tue',
            'tuesday',
            'tumblr',
            'turpis',
            'twiter',
            'twitterfeed',
            'txmt',
            'txt',
            'txtspk',
            'typeerror',
            'typekit',
            'typeless',
            'typeof',
            'typoe',
            'tz',
            'uas',
            'ubb',
            'ubuntu',
            'ucfirst',
            'ucwords',
            'uglify',
            'uhash',
            'uhoh',
            'ui',
            'uid',
            'uis',
            'uk',
            'ul',
            'ullamcorper',
            'ultrices',
            'ultricies',
            'un',
            'unachievable',
            'unadvisable',
            'unary',
            'unassign',
            'unauthorized',
            'unban',
            'unbannable',
            'unbanned',
            'unbanning',
            'unbans',
            'unblind',
            'unblockable',
            'unboxvideo',
            'unbundle',
            'unbundled',
            'uncached',
            'uncategorised',
            'uncheck',
            'unchecking',
            'unclosed',
            'uncomment',
            'uncommented',
            'uncommenting',
            'unconfigurable',
            'unconfigured',
            'undeclare',
            'une',
            'uneditable',
            'unengaged',
            'unevaluated',
            'unexperienced',
            'unfeasibly',
            'unfloated',
            'unfound',
            'unhandled',
            'unhashed',
            'unicef',
            'unicode',
            'unificiation',
            'unindexable',
            'uninstallation',
            'uniqify',
            'unix',
            'unixodbc',
            'unlink',
            'unmanaged',
            'unmatchable',
            'unmoderated',
            'unmonitor',
            'unordered',
            'unparsed',
            'unpause',
            'unpausing',
            'unprefixed',
            'unprocessable',
            'unrate',
            'unrealisable',
            'unrecognized',
            'unreferenced',
            'unreliabilities',
            'unrendered',
            'unsatisfiable',
            'unselect',
            'unsets',
            'unsetting',
            'unsilence',
            'unsleeping',
            'unslider',
            'unsuspend',
            'untar',
            'untarred',
            'unterminated',
            'untick',
            'unticked',
            'unticking',
            'untrustable',
            'untrusted',
            'unvetted',
            'unvisited',
            'unvote',
            'upbars',
            'upgradedatabaseifrequired',
            'upgrader',
            'upgraders',
            'uploader',
            'uploaders',
            'uptime',
            'upwork',
            'uri',
            'uris',
            'url',
            'urlencode',
            'urlencoded',
            'urlencodes',
            'urls',
            'urna',
            'userbases',
            'usergroup',
            'usergroupid',
            'usergroups',
            'userguide',
            'userid',
            'usermappr',
            'usermeta',
            'usingpage',
            'usps',
            'usr',
            'ut',
            'utf',
            'util',
            'uuid',
            'validator',
            'validators',
            'valuators',
            'valuemap',
            'vampirism',
            'varius',
            've',
            'velit',
            'vely',
            'vera',
            'verboser',
            'verdana',
            'verisign',
            'vestibulum',
            'vfm',
            'vforums',
            'vfs',
            'vid',
            'videogames',
            'videoid',
            'vienna',
            'viewable',
            'viewport',
            'viewports',
            'vimeo',
            'virtualdomains',
            'virtualed',
            'virtualised',
            'virtualising',
            'vivamus',
            'viverra',
            'vm',
            'vms',
            'voiceover',
            'volutpat',
            'voteable',
            'vpn',
            'vpns',
            'vulputate',
            'walkthrough',
            'walter',
            'wampserver',
            'warner',
            'watchlist',
            'waterfox',
            'watson',
            'wav',
            'wb',
            'wbb',
            'wcag',
            'webadmin',
            'webapp',
            'webclient',
            'webclip',
            'webclipicon',
            'webcrawlers',
            'webdav',
            'webgl',
            'webhost',
            'webhosting',
            'webhosts',
            'webkit',
            'webm',
            'webmail',
            'webmatrix',
            'webp',
            'webpage',
            'webpages',
            'webpi',
            'webring',
            'webroot',
            'webrtc',
            'websafe',
            'webserver',
            'webstandards',
            'webster',
            'webtv',
            'weburl',
            'wednesday',
            'wedway',
            'wget',
            'whatsmore',
            'whatsnew',
            'wheremap',
            'safelist',
            'safelisted',
            'safelisting',
            'safelists',
            'blocklist',
            'blocklisted',
            'blocklisting',
            'blocklists',
            'whitepaper',
            'whitespace',
            'whiz',
            'whois',
            'whoopsie',
            'wi',
            'wifi',
            'wikipedia',
            'wil',
            'wildcarded',
            'wildcarding',
            'willl',
            'wilson',
            'wincache',
            'wincachegrind',
            'winmerge',
            'winscp',
            'winston',
            'wireframes',
            'wishlist',
            'withheading',
            'withinweek',
            'withname',
            'withparamname',
            'withprettyname',
            'wix',
            'wizable',
            'wjs',
            'wmv',
            'woltlab',
            'woorank',
            'wordfilter',
            'wordpress',
            'worke',
            'workspace',
            'worldcitiespop',
            'worldgazetteer',
            'wowbb',
            'wp',
            'writeable',
            'writeup',
            'wrt',
            'www',
            'wysiwyg',
            'wysiwyg\'d',
            'xbox',
            'xcache',
            'xchanged',
            'xcode',
            'xdebug',
            'xfbml',
            'xhr',
            'xhtml',
            'xkcd',
            'xmb',
            'xml',
            'xmlconfigscreen',
            'xmlentities',
            'xmlhttprequest',
            'xmlns',
            'xmlrpc',
            'xvf',
            'xvid',
            'yaml',
            'ychanged',
            'yeehaw',
            'york',
            'yourwebsite',
            'youtu',
            'youtube',
            'yt',
            'yy',
            'yyy',
            'yyyy',
            'yyyyynn',
            'zend',
            'zipcode',
            'zipcodes',
            'zlib',
            'zoho',
            'zombiism',
            'zoneeditorpanel',
            'zoneeditorscreen',
            'zxxxxx',
            'chmoddable',
            'disconsider',
            'dataattr',
            'aseem',
            'kishore',
            'sublicense',
            'douglas',
            'crockford',
            'unescaped',
            'reviver',
            'stringify',
            'tojson',
            'crokford',
            'unbox',
            'arraybuffers',
            'capitalization',
            'normalized',
            'getresponseheader',
            'issymbol',
            'definepolyfill',
            'fromindex',
            'keyboardevent',
            'searchparams',
            'urlutils',
            'htmlanchorelement',
            'urlsearchparams',
            'targetable',
            'yandex',
            'infinitum',
            'algorithmically',
            'overcomplex',
            'specced',
            'arity',
            'translatability',
            'directioning',
            'evaluable',
            'updatable',
            'choosable',
            'writability',
            'disconsidering',
            'actualise',
            'unescape',
            'unescaping',
            'storable',
            'determinative',
            'valignment',
            'submissionflow',
            'undef',
            'whitelisted',
            'uniqid',
            'bak',
            'crowdsourced',
            'mjs',
        ];

        $spell_link = spellcheck_initialise('en_GB');
        add_spellchecker_words_temp($spell_link, $okay_words);

        // Check disk files...

        $paths = [
            '' => 'php',

            'sources' => 'php',
            'sources_custom' => 'php',

            '_tests/cmstest' => 'php',
            '_tests/tests' => 'php',
            'data' => 'php',
            'data_custom' => 'php',
            'mobiquo/api' => 'php',
            'mobiquo/tests' => 'php',
            'mobiquo/include' => 'php',
            'uploads' => 'php',

            'pages' => 'php',
            'site' => 'php',
            'docs/pages' => 'php',
            'docs' => 'md',
            'forum' => 'php',
            'adminzone' => 'php',
            'cms' => 'php',
            'buildr' => 'php',

            'themes/default/css' => 'css',
            'themes/default/css_custom' => 'css',

            'themes/default' => 'js',

            'lang/EN' => 'ini',
            'lang_custom/EN' => 'ini',

            'site/pages/comcode/EN' => 'txt',
            'pages/comcode/EN' => 'txt',
            'forum/comcode/EN' => 'txt',
            'adminzone/comcode/EN' => 'txt',
            'cms/comcode/EN' => 'txt',

            'site/pages/comcode_custom/EN' => 'txt',
            'pages/comcode_custom/EN' => 'txt',
            'forum/comcode_custom/EN' => 'txt',
            'buildr/comcode_custom/EN' => 'txt',
            'docs/pages/comcode_custom/EN' => 'txt',

            'text/EN' => 'txt',
            'text_custom/EN' => 'txt',
            'data/modules/cms_comcode_pages/EN' => 'txt',
            '_tests' => 'txt',
        ];
        foreach ($paths as $_path => $_ext) {
            $recurse = ($_path != '');
            $files = get_directory_contents(get_file_base() . '/' . $_path, $_path, IGNORE_ALIEN | IGNORE_SHIPPED_VOLATILE | IGNORE_CUSTOM_DIRS | IGNORE_UNSHIPPED_VOLATILE | IGNORE_FLOATING, $recurse, true, [$_ext]);

            foreach ($files as $path) {
                // Exceptions
                $exceptions = array_merge(list_untouchable_third_party_directories(), [
                ]);
                if (preg_match('#^(' . implode('|', $exceptions) . ')/#', $path) != 0) {
                    continue;
                }
                $exceptions = array_merge(list_untouchable_third_party_files(), [
                ]);
                if (in_array($path, $exceptions)) {
                    continue;
                }

                $ext = get_file_extension($path);
                if ($ext != $_ext) {
                    continue;
                }

                if (($this->only !== null) && ($this->only != $path)) {
                    continue;
                }

                if ($path == 'data_custom/errorlog.php') {
                    continue;
                }

                if ($this->debug) {
                    var_dump($path);
                }

                // Load and pre-process into plain text
                $c = cms_file_get_contents_safe(get_file_base() . '/' . $path);
                switch ($ext) {
                    case 'ini':
                        $c = str_replace('\n', ' ', $c);
                        $c = clean_simple_html_for_spellcheck($c);
                        break;

                    case 'htm':
                    case 'html':
                        $c = clean_simple_html_for_spellcheck($c);
                        break;

                    case 'md':
                    case 'txt':
                        if (in_array($path, [
                            'docs/pages/comcode_custom/EN/tut_addon_index.txt',
                            'text/EN/synonyms.txt',
                            'docs/LICENSE.md',
                            'docs/THANKS.md',
                            'docs/CODE_OF_CONDUCT.md',
                        ])) {
                            continue 2;
                        }

                        $c = preg_replace('#\[tt.*\[/tt\]#Us', '', $c);
                        $c = preg_replace('#<kbd.*</kbd>#Us', '', $c);

                        $c = clean_comcode_for_spellcheck($c);

                        break;

                    case 'js':
                    case 'css':
                    case 'php':
                        // We just want the code comments...

                        $c = preg_replace('#https?://[^\s\'"]*#', '', $c);

                        $_c = '';
                        $matches = [];

                        if ($ext != 'css') {
                            $num_matches = preg_match_all('#[^:]//(.*)#', $c, $matches);
                            for ($i = 0; $i < $num_matches; $i++) {
                                $_c .= ' ' . $matches[1][$i];
                            }
                        }

                        $num_matches = preg_match_all('#/\*(.*)\*/#Um', $c, $matches);
                        for ($i = 0; $i < $num_matches; $i++) {
                            $_c .= ' ' . $matches[1][$i];
                        }

                        $_c = preg_replace('#\w*_\w*#', '', $_c); // Not useful for checking code comments, we're often just referencing code terms
                        $_c = preg_replace('#\$\w+#', '', $_c); // Not useful for checking code comments, we're often just referencing variables

                        $c = $_c;

                        break;
                }

                $c = preg_replace('#`[^`\n]*`#', '', $c); // Protected

                $misspellings = run_spellcheck($c, null, false/*would be too slow and is not needed*/, false, false);
                if ($misspellings === null) {
                    $misspellings = [];
                    $this->assertTrue(false, 'The spell check failed to run.');
                }
                foreach (array_keys($misspellings) as $word) {
                    $this->assertTrue(false, 'Misspelling: ' . $word . ' (' . $path . ')');
                }

                if ($ext != 'ini') {
                    $manual_check_ob->check($this, $path, null, $c, false);
                }
            }
        }

        // Now check addon descriptions...

        $addons = find_all_hook_obs('systems', 'addon_registry', 'Hook_addon_registry_');
        foreach ($addons as $addon_name => $ob) {
            if (($this->only !== null) && ($this->only != $addon_name) && ($this->only != '(addons)')) {
                continue;
            }

            $this->assertTrue(method_exists($ob, 'get_description'), 'Missing get_description for ' . $addon_name);

            $c = $ob->get_description();

            $c = clean_comcode_for_spellcheck($c);

            $misspellings = run_spellcheck($c, null, false/*would be too slow and is not needed*/, false, false);
            if ($misspellings === null) {
                $misspellings = [];
                $this->assertTrue(false, 'The spell check failed to run.');
            }
            foreach (array_keys($misspellings) as $word) {
                $this->assertTrue(false, 'Misspelling: ' . $word . ' (' . $addon_name . ' addon description)');
            }
        }

        spellchecker_shutdown();
    }
}
